/*
 * mag-board V1.3 - no WiFi
 *
 */
 

#include <EasyBuzzer.h>
#include "stdio.h"
#include "driver/ledc.h"
#include "driver/pcnt.h"
#include "soc/timer_group_struct.h"
#include "soc/timer_group_reg.h"
#include "soc/rtc_wdt.h"


#define LCD_ON
#define LCD_I2C_ON

#ifdef LCD_I2C_ON
#define I2C_SDA 21
#define I2C_SCL 22
#include <Wire.h>
#include <LiquidCrystal_PCF8574.h>
LiquidCrystal_PCF8574 lcd(0x27);
#endif

byte signal_0[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1B};
byte signal_1[] = {0x00,0x00,0x00,0x00,0x00,0x18,0x00,0x1B};
byte signal_2[] = {0x00,0x00,0x00,0x00,0x03,0x1B,0x00,0x1B};
byte signal_3[] = {0x00,0x00,0x00,0x18,0x18,0x18,0x00,0x1B};
byte signal_4[] = {0x00,0x00,0x03,0x1B,0x1B,0x1B,0x00,0x1B};
byte signal_5[] = {0x00,0x18,0x18,0x18,0x18,0x18,0x00,0x1B};
byte signal_6[] = {0x03,0x1B,0x1B,0x1B,0x1B,0x1B,0x00,0x1B};  

uint32_t max_value = 500;                                          //max value for graph scale and buzzer this can be set by app or 2. buttons
int  last_strength = 0;

#define BUZZER        17 
#define NULL_BUTTON   25
#define PB1_BUTTON    5
#define PB2_BUTTON    18

//---------------FREQ COUNTING COMMON------------------
// FREQ1 Counter
#define UNIT_FREQ           PCNT_UNIT_0
#define CHANNEL_FREQ        PCNT_CHANNEL_0
#define INPUT_PIN_FREQ      GPIO_NUM_27
#define CTRL_PIN_FREQ       GPIO_NUM_34
#define OUTPUT_PIN_FREQ     GPIO_NUM_33

bool              flag_freq       = true;
int16_t           counterFreq     = 0;
uint32_t          multiplierFreq  = 0;
volatile uint32_t frequency       = 0;

// FREQ2 Counter
#define UNIT_FREQ2          PCNT_UNIT_1
#define CHANNEL_FREQ2       PCNT_CHANNEL_0
#define INPUT_PIN_FREQ2     GPIO_NUM_26
#define CTRL_PIN_FREQ2      GPIO_NUM_35
#define OUTPUT_PIN_FREQ2    GPIO_NUM_32

bool              flag_freq2      = true;
int16_t           counterFreq2    = 0;
uint32_t          multiplierFreq2 = 0;
volatile uint32_t frequency2      = 0;

volatile uint32_t frequency_diff  = 0;

#define MAX_COUNTER_VAL         10000
#define IN_BOARD_LED          GPIO_NUM_2

char            buf[32];

esp_timer_create_args_t create_args;
esp_timer_create_args_t create_args2;

esp_timer_handle_t timer_freq;
esp_timer_handle_t timer_freq2;

portMUX_TYPE timerMux = portMUX_INITIALIZER_UNLOCKED;


void onZeroButtonPressed() {
  frequency_diff = frequency2 - frequency;
}

//----------------------------------------------------------------------------------------
void setup() {

  rtc_wdt_protect_off();
  rtc_wdt_disable();
  
  Serial.begin(115200);
  setCpuFrequencyMhz(80);        // set core frequency
  uint32_t Freq = 0;
  Freq = getCpuFrequencyMhz();
  Serial.print("CPU Freq = ");
  Serial.print(Freq);
  Serial.println(" MHz");
  Freq = getXtalFrequencyMhz();
  Serial.print("XTAL Freq = ");
  Serial.print(Freq);
  Serial.println(" MHz");
  Freq = getApbFrequency();
  Serial.print("APB Freq = ");
  Serial.print(Freq);
  Serial.println(" Hz");
  

#ifdef LCD_I2C_ON
  Wire.begin(I2C_SDA, I2C_SCL);
  lcd.setBacklight(255);
#endif

#if defined LCD_ON || defined LCD_I2C_ON
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print("MAG BOARD V1.3- ");
  lcd.setCursor(0, 1);
  lcd.print(" fgsensors.com ");
  delay(5000);
  lcd.setCursor(0, 0);
  lcd.print("                ");
  lcd.setCursor(0, 1);
  lcd.print("                ");
#endif

  init_measurements();
  lcd.createChar(1, signal_0);                        //signal bar graphic
  lcd.createChar(2, signal_1);
  lcd.createChar(3, signal_2);
  lcd.createChar(4, signal_3);
  lcd.createChar(5, signal_4);
  lcd.createChar(6, signal_5);
  lcd.createChar(7, signal_6);

  pinMode(NULL_BUTTON, INPUT);                      // null button
  pinMode(PB1_BUTTON, INPUT);                       // + button
  pinMode(PB2_BUTTON, INPUT);                       // - button
  pinMode(BUZZER, OUTPUT);                          // buzzer
  
  EasyBuzzer.setPin(BUZZER);
  EasyBuzzer.stopBeep();
}

//------------------------------------------------------------------
static void IRAM_ATTR pcnt_intr_handler(void *arg)
{
  portENTER_CRITICAL_ISR(&timerMux);
  int i;

  for(i = 0; i < PCNT_UNIT_MAX; i++) {
      if(PCNT.int_st.val & (BIT(PCNT_UNIT_0))) {
          PCNT.int_clr.val = BIT(PCNT_UNIT_0);
          multiplierFreq++;
      } else if(PCNT.int_st.val & (BIT(PCNT_UNIT_1))) {
          PCNT.int_clr.val = BIT(PCNT_UNIT_1);
          multiplierFreq2++;        
      } 
    }
    portEXIT_CRITICAL_ISR(&timerMux);
}

//------------------------------------------------------------------
void freq_control(gpio_num_t pin, pcnt_unit_t unit, bool &flag, int16_t *value)
{
  gpio_set_level(pin, 0);
  pcnt_get_counter_value(unit, value);
  flag = true;
}

//------------------------------------------------------------------
void freq1_control(void *p)
{
  freq_control(OUTPUT_PIN_FREQ, UNIT_FREQ, flag_freq, &counterFreq);
}

//------------------------------------------------------------------
void freq2_control(void *p)
{
  freq_control(OUTPUT_PIN_FREQ2, UNIT_FREQ2, flag_freq2, &counterFreq2);
}

//------------------------------------------------------------------

void init_gpio_out(gpio_num_t pin)
{
  gpio_pad_select_gpio(pin);
  gpio_set_direction(pin, GPIO_MODE_OUTPUT);
}

//------------------------------------------------------------------
void init_counter(int input_pin, int ctrl_pin, pcnt_unit_t unit, pcnt_channel_t channel, gpio_num_t gpio_out)
{
  pcnt_config_t pcnt_config = { };
  pcnt_config.pulse_gpio_num = input_pin;
  pcnt_config.ctrl_gpio_num = ctrl_pin;
  pcnt_config.unit = unit;
  pcnt_config.channel = channel;
  pcnt_config.counter_h_lim = MAX_COUNTER_VAL;
  pcnt_config.pos_mode = PCNT_COUNT_INC;
  pcnt_config.neg_mode = PCNT_COUNT_INC;
  pcnt_config.lctrl_mode = PCNT_MODE_DISABLE;
  pcnt_config.hctrl_mode = PCNT_MODE_KEEP;

  pcnt_unit_config(&pcnt_config);

  pcnt_event_enable(unit, PCNT_EVT_H_LIM);
  pcnt_counter_pause(unit);
  pcnt_counter_clear(unit);
  
  pcnt_intr_enable(unit);
  pcnt_counter_resume(unit);
  
  init_gpio_out(gpio_out);
}

//------------------------------------------------------------------
void init_measurements()
{
  pcnt_isr_register(pcnt_intr_handler, NULL, 0, NULL);
  
  init_counter(INPUT_PIN_FREQ, CTRL_PIN_FREQ, UNIT_FREQ, CHANNEL_FREQ, OUTPUT_PIN_FREQ);
  init_counter(INPUT_PIN_FREQ2, CTRL_PIN_FREQ2, UNIT_FREQ2, CHANNEL_FREQ2, OUTPUT_PIN_FREQ2);

 
  create_args.callback = freq1_control;
  esp_timer_create(&create_args, &timer_freq);

  create_args2.callback = freq2_control;
  esp_timer_create(&create_args2, &timer_freq2);

  gpio_matrix_in(INPUT_PIN_FREQ, SIG_IN_FUNC226_IDX, false);
  gpio_matrix_in(INPUT_PIN_FREQ2, SIG_IN_FUNC226_IDX, false);
}

//------------------------------------------------------------------
void signal_strength(int sig_str)
{
  lcd.setCursor(13,0);
  switch (sig_str) {
        case 0 ... 10:                              
          lcd.write(1);lcd.write(1);lcd.write(1);
        break;
         case 11 ... 24:                             
           lcd.write(2);lcd.write(1);lcd.write(1);
        break;
          case 25 ... 37:                              
           lcd.write(3);lcd.write(1);lcd.write(1);
        break;
         case 38 ... 50:                            
           lcd.write(3);lcd.write(4);lcd.write(1);
        break;
         case 51 ... 64:                             
          lcd.write(3);lcd.write(5);lcd.write(1);
       break;
         case 65 ... 77:                              
           lcd.write(3);lcd.write(5);lcd.write(6);
       break;
         default:                             
          lcd.write(3);lcd.write(5);lcd.write(7);
        break;
       }
}

//------------------------------------------------------------------
void loop()
{
  rtc_wdt_feed();
//  EasyBuzzer.update();
  bool flag1 = flag_freq, flag2 = flag_freq2;

  if (flag1 == true)
  {
    flag_freq = false;
    frequency = (counterFreq + (multiplierFreq * MAX_COUNTER_VAL)) * 5;
    multiplierFreq = 0;
  }
  if (flag2 == true) {
    flag_freq2 = false;
    frequency2 = (counterFreq2 + (multiplierFreq2 * MAX_COUNTER_VAL)) * 5;
    multiplierFreq2 = 0;
  }

  if (flag1 || flag2) {
    delay (100);
  }
  if (digitalRead(NULL_BUTTON) == LOW) {
    onZeroButtonPressed();
  }

  if (digitalRead(PB1_BUTTON) == LOW) {
    max_value += 100;
  }

  if (digitalRead(PB2_BUTTON) == LOW) {
    if (max_value > 100) 
      max_value -= 100;
  }
  
  if (flag1) {
    pcnt_counter_clear(UNIT_FREQ);
   esp_timer_start_once(timer_freq, 100000);
    gpio_set_level(OUTPUT_PIN_FREQ, 1);
  }
  if (flag2) {
    pcnt_counter_clear(UNIT_FREQ2);
    esp_timer_start_once(timer_freq2, 100000);
    gpio_set_level(OUTPUT_PIN_FREQ2, 1);
  }

  int freq_diff = frequency2-frequency-frequency_diff;
  int abs_freq_diff = abs(freq_diff);
  signal_strength(abs_freq_diff * 100/ max_value);

   if (abs_freq_diff > 10) {
    if ((last_strength / 50) != (abs_freq_diff / 50)) {
      EasyBuzzer.beep(abs_freq_diff / 5);
    }
   } else {
      if (last_strength > 10) {
        EasyBuzzer.stopBeep();
      }
   }
   last_strength = abs_freq_diff;
 
#if defined LCD_ON || defined LCD_I2C_ON
    lcd.setCursor(0, 0);
    sprintf(buf, "U%6d T%4d", frequency, max_value);
    lcd.print(buf);
    
    lcd.setCursor(0, 1);
    sprintf(buf, "L%6d D%5d   ", frequency2,freq_diff);
    lcd.print(buf);
#endif
}
